package kubernetes

required_k8s_labels = ["app.kubernetes.io/name", "app.kubernetes.io/instance", "app.kubernetes.io/version", "app.kubernetes.io/component", "app.kubernetes.io/part-of", "app.kubernetes.io/managed-by"]
labels {
	input.metadata.labels["app.kubernetes.io/name"]
	input.metadata.labels["app.kubernetes.io/instance"]
	input.metadata.labels["app.kubernetes.io/version"]
	input.metadata.labels["app.kubernetes.io/component"]
	input.metadata.labels["app.kubernetes.io/part-of"]
	input.metadata.labels["app.kubernetes.io/managed-by"]
}

required_sbu_annotations = ["sbu.vmware.com/owner", "sbu.vmware.com/sguid", "sbu.vmware.com/src_url", "sbu.vmware.com/stage", "sbu.vmware.com/sku"]
sbu_annotations {
	input.metadata.annotations["sbu.vmware.com/owner"]
	input.metadata.annotations["sbu.vmware.com/sguid"]
	input.metadata.annotations["sbu.vmware.com/src_url"]
	input.metadata.annotations["sbu.vmware.com/stage"]
	input.metadata.annotations["sbu.vmware.com/sku"]
}

# === Kubernetes Labels ===
resource_missing_tags[msg] {
	not labels
	msg = sprintf("%s %s must include Kubernetes recommended labels %s: https://kubernetes.io/docs/concepts/overview/working-with-objects/common-labels/#labels", [name, input.kind, required_k8s_labels])
}

deny_resource_missing_tags[msg]{
  not array_contains(warning_mode, "resource_missing_tags")
  resource_missing_tags[_] != []
  msg := resource_missing_tags[_]
}

warn_resource_missing_tags[msg]{
  array_contains(warning_mode, "resource_missing_tags")
  resource_missing_tags[_] != []
  msg := resource_missing_tags[_]
}

# === Kubernetes SBU Annotations ===
resource_missing_annotations[msg] {
	not sbu_annotations
	msg = sprintf("%s %s must include SBU required annotations %s: https://confluence.carbonblack.local/display/cbengx/SBU+AWS+Tagging+Policy", [name, input.kind, required_sbu_annotations])
}

deny_resource_missing_annotations[msg]{
  not array_contains(warning_mode, "resource_missing_annotations")
  resource_missing_annotations[_] != []
  msg := resource_missing_annotations[_]
}

warn_resource_missing_annotations[msg]{
  array_contains(warning_mode, "resource_missing_annotations")
  resource_missing_annotations[_] != []
  msg := resource_missing_annotations[_]
}

# === Kubernetes SBU Annotation Email ===
deployment_bad_owner_annotation[msg] {
	# check if "sbu.vmware.com/owner" belongs to the set
	input.metadata.annotations["sbu.vmware.com/owner"]
	# if it does, check that it's a valid VMware email address
	val := input.metadata.annotations["sbu.vmware.com/owner"]
	not re_match(`^([\.a-zA-Z0-9+_-]+)(@vmware\.com|@(.*).vmware\.com)$`, val)
	msg = sprintf("%s %s owner annotation is [%s] which is not a valid email: https://confluence.carbonblack.local/display/cbengx/SBU+AWS+Tagging+Policy", [name, input.kind, val])
}

deny_deployment_bad_owner_annotation[msg]{
  not array_contains(warning_mode, "deployment_bad_owner_annotation")
  deployment_bad_owner_annotation[_] != []
  msg := deployment_bad_owner_annotation[_]
}

warn_deployment_bad_owner_annotation[msg]{
  array_contains(warning_mode, "deployment_bad_owner_annotation")
  deployment_bad_owner_annotation[_] != []
  msg := deployment_bad_owner_annotation[_]
}

# === Kubernetes SBU Annotation sguid ===
resource_bad_sguid_annotation[msg] {
	# check if "sbu.vmware.com/sguid" belongs to the set
	input.metadata.annotations["sbu.vmware.com/sguid"]
	# if it does, check that it's a valid sguid value
	val := input.metadata.annotations["sbu.vmware.com/sguid"]
	val_array := split(lower(val),":")
	array_contains(val_array, "dummy")
	msg = sprintf("%s %s sguid annotation is [%s] which is not a valid sguid value it can not have DUMMY or NONE value: https://confluence.carbonblack.local/display/cbengx/SBU+AWS+Tagging+Policy", [name, input.kind, val])
}

resource_bad_sguid_annotation[msg] {
	# check if "sbu.vmware.com/sguid" belongs to the set
	input.metadata.annotations["sbu.vmware.com/sguid"]
	# if it does, check that it's a valid sguid value
	val := input.metadata.annotations["sbu.vmware.com/sguid"]
	val_array := split(lower(val),":")
	array_contains(val_array, "none")
	msg = sprintf("%s %s sguid annotation is [%s] which is not a valid sguid value it can not have DUMMY or NONE value: https://confluence.carbonblack.local/display/cbengx/SBU+AWS+Tagging+Policy", [name, input.kind, val])
}

deny_deployment_sguid_annotation[msg]{
  not array_contains(warning_mode, "resource_bad_sguid_annotation")
  resource_bad_sguid_annotation[_] != []
  msg := resource_bad_sguid_annotation[_]
}

warn_deployment_sguid_annotation[msg]{
  array_contains(warning_mode, "resource_bad_sguid_annotation")
  resource_bad_sguid_annotation[_] != []
  msg := resource_bad_sguid_annotation[_]
}

# === Kubernetes SBU Annotation sguid ===
resource_bad_sguid_label[msg] {
	# check if "sbu.vmware.com/sguid" belongs to the set
	input.metadata.labels["sbu.vmware.com/sguid"]
	# if it does, check that it's a valid sguid value
	val := input.metadata.labels["sbu.vmware.com/sguid"]
	val_array := split(lower(val),".")
	array_contains(val_array, "dummy")
	msg = sprintf("%s %s sguid annotation is [%s] which is not a valid sguid value it can not have DUMMY or NONE value: https://confluence.carbonblack.local/display/cbengx/SBU+AWS+Tagging+Policy", [name, input.kind, val])
}

resource_bad_sguid_label[msg] {
	# check if "sbu.vmware.com/sguid" belongs to the set
	input.metadata.labels["sbu.vmware.com/sguid"]
	# if it does, check that it's a valid sguid value
	val := input.metadata.labels["sbu.vmware.com/sguid"]
	val_array := split(lower(val),".")
	array_contains(val_array, "none")
	msg = sprintf("%s %s sguid label is [%s] which is not a valid sguid value it can not have DUMMY or NONE value: https://confluence.carbonblack.local/display/cbengx/SBU+AWS+Tagging+Policy", [name, input.kind, val])
}

deny_resource_bad_sguid_label[msg]{
  not array_contains(warning_mode, "resource_bad_sguid_label")
  resource_bad_sguid_label[_] != []
  msg := resource_bad_sguid_label[_]
}

warn_resource_bad_sguid_label[msg]{
  array_contains(warning_mode, "resource_bad_sguid_label")
  resource_bad_sguid_label[_] != []
  msg := resource_bad_sguid_label[_]
}