package mosaicv2

import data.config

# This rego policy file is for a Blast Radius check utilized by the Mosaic 2.0 team. It's utilized by the
# Infra-Charts repository [https://gitlab.bit9.local/mcs2/infrastructure-charts/-/blob/master/Dockerfile]
# To use this policy run conftest with `--namespace mosaicv2`, it also utilizes a special `forceDeploy` flag
# in the config file to override the tests instead of the pattern set in the other policy files in this repo.

################################################################################
##                           Helpers/Exceptions                               ##
################################################################################

# Helper function for checking array for containing values
array_contains(arr, elem) {
  arr[_] = elem
}

# Exception statement
exception[rules] {
  config.forceDeploy == "true"

  rules := [
    "resource_deletion",
    "planned_deletions",
    "resource_recreation",
    "planned_recreations",
    "prohibited_resource"
  ]
}

# List of resources where both update & delete actions are allowed
allow_changes = [
	"aws_rds_cluster_parameter_group",
	"aws_db_parameter_group",
	"aws_lambda_layer_version",
	"time_sleep",
	"aws_rds_cluster_instance",
	"aws_iam_policy"
]

################################################################################
## Test #0 - Check for Deletions                                              ##
## Calculates a Blast Radius based on the number of DELETE actions            ##
################################################################################

# Maximum number of deletions allowed
allowed_deletions = 0

# Checks for resources being deleted, resource deletion is determined by the "actions":[] field
# containing ONLY the "delete" action
get_deletions = resources {
	all := input.resource_changes
	resources := [ res |
	    res := all[_];
	    not array_contains(allow_changes, res.type)
	    res.change.actions == ["delete"]
	]
}

# Warning output for each resource being deleted
warn_resource_deletion[reason] {
	deletions := get_deletions
	reason := sprintf("'%v' is being deleted in the terraform plan", [deletions[_].address])
}

# Calculates number of deletions
deny_planned_deletions[reason] {
	score := count(get_deletions)
	score > allowed_deletions
	reason := sprintf("Detected %v delete action(s) in terraform plan, number allowed %v \nPlease see our Confluence document for more information https://confluence.eng.vmware.com/x/qVeFOQ", [score, allowed_deletions])
}

################################################################################
## Test #1 - Check for Recreations                                            ##
## Calculates a Blast Radius based on the number of DELETE, CREATE actions    ##
################################################################################

# Maximum number of recreations allowed
allowed_recreations = 0

# Checks for resources being recreated, resource recreation is determined by the "actions":[] field
# containing BOTH the "create" and "delete" actions
get_recreations = resources {
	all := input.resource_changes
	resources := [ res |
	    res := all[_];
	    not array_contains(allow_changes, res.type)
	    res.change.actions[_] == "delete";
	    res.change.actions[_] == "create"
	]
}

# Warning output for each resource being recreated
warn_resource_recreation[reason] {
	recreations := get_recreations
	reason := sprintf("'%v' is being recreated in the terraform plan", [recreations[_].address])
}

# Calculates number of recreations
deny_planned_recreations[reason] {
	score := count(get_recreations)
	score > allowed_recreations
	reason := sprintf("Detected %v recreate action(s) in terraform plan, number allowed %v \nPlease see our Confluence document for more information https://confluence.eng.vmware.com/x/qVeFOQ", [score, allowed_recreations])
}

################################################################################
## Test #2 -  Check for Prohibited Resources                                  ##
## Check for denied resource types created in the plan based on the denylist  ##
################################################################################

# List of denied resource types
denylist = [
	"aws_cloudfront_distribution",
	"aws_cloudfront_key_group",
	"aws_cloudfront_public_key"
]

# Checks whether the plan will cause resources with certain prefixes to be created
warn_prohibited_resource[reason] {
	some i
	# Get all resource types with planned changes
	all := [ name | name := input.resource_changes[_].type ]
	# For i in all, check if in denylist
	startswith(all[i], denylist[_])
	reason := sprintf("Prohibited Terraform resource detected: '%v'", [all[i]])
}

################################################################################
## Test #3 - Check for Recreations on RDS Read Replicas                       ##
## Makes sure that all replicas are not destroyed, allows changes otherwise   ##
################################################################################

# Deny plan if the number of recreate/delete actions on "aws_rds_cluster_instance" is >= number of replicas
deny_rds_read_replica_blanket_delete[reason] {
	all := input.resource_changes
	total_changes := count([ res |
	    res := all[_];
	    res.type == "aws_rds_cluster_instance"
	])
	not total_changes == 0
	recreate := count([ res |
	    res := all[_];
	    res.type == "aws_rds_cluster_instance"
	    res.change.actions[_] == "delete";
	    res.change.actions[_] == "create"
	])
	delete := count([ res |
	    res := all[_];
	    res.type == "aws_rds_cluster_instance"
	    res.change.actions == ["delete"]
	])
	destructive_actions := recreate + delete
	destructive_actions >= total_changes
	reason := sprintf("Detected '%v' destructive action(s) on aws_rds_cluster_instance types ('%v' recreated, '%v' deleted) which would result in all replicas being destroyed \nPlease see our Confluence document for more information https://confluence.eng.vmware.com/x/qVeFOQ", [destructive_actions, recreate, delete])
}