package main

logging_type := "audit"

rds_instances[r] {
	r := input.resource_changes[_]
	r.type == "aws_db_instance"
}

################################################################################
## Test Cases                                                                 ##
################################################################################

# === Rule to restrict RDS which are not MultiAZ ===
rds_multi_az[reason] {
	r := rds_instances[_]
	not r.change.after.multi_az
	reason := sprintf(
		"%s: Database is not MultiAZ. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
		[r.name]
	)
}

deny_rds_multi_az[msg]{
	not array_contains(warning_mode, "rds_multi_az")
	rds_multi_az[_] != []
	msg := rds_multi_az[_]
}

warn_rds_multi_az[msg]{
	array_contains(warning_mode, "rds_multi_az")
	rds_multi_az[_] != []
	msg := rds_multi_az[_]
}

# === Rule to restrict RDS without audit logging ===
rds_audit_logging_disabled[reason] {
	r := rds_instances[_]
	not array_contains(r.change.after.enabled_cloudwatch_logs_exports, logging_type)
	reason := sprintf(
		"%s: Database doesn't not have Audit logging enabled. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
		[r.name]
	)
}

deny_rds_audit_logging_disabled[msg]{
	not array_contains(warning_mode, "rds_audit_logging_disabled")
	rds_audit_logging_disabled[_] != []
	msg := rds_audit_logging_disabled[_]
}

warn_rds_audit_logging_disabled[msg]{
	array_contains(warning_mode, "rds_audit_logging_disabled")
	rds_audit_logging_disabled[_] != []
	msg := rds_audit_logging_disabled[_]
}

# === Deny if RDS backup not set ===
rds_bkp[reason] {
	r := rds_instances[_]
	not r.change.after.backup_retention_period
	reason := sprintf("%s: RDS Backup is not set. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.", [r.address])
}

deny_rds_bkp[msg]{
	not array_contains(warning_mode, "rds_bkp")
	rds_bkp[_] != []
	msg := rds_bkp[_]
}

warn_rds_bkp[msg]{
	array_contains(warning_mode, "rds_bkp")
	rds_bkp[_] != []
	msg := rds_bkp[_]
}

# === Deny if RDS backup retention is less than 30 days ===
rds_bkp_period[reason] {
	r := rds_instances[_]
	r.change.after.backup_retention_period < 30
	reason := sprintf("%s: RDS Backup Retention less than 30 days is not allowed. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.", [r.address])
}

deny_rds_bkp_period[msg]{
	not array_contains(warning_mode, "rds_bkp_period")
	rds_bkp_period[_] != []
	msg := rds_bkp_period[_]
}

warn_rds_bkp_period[msg]{
	array_contains(warning_mode, "rds_bkp_period")
	rds_bkp_period[_] != []
	msg := rds_bkp_period[_]
}

# === Deny if RDS deletion protection is not enabeled ===
rds_del_protection[reason] {
	r := rds_instances[_]
	not r.change.after.deletion_protection == true
	reason := sprintf("%s: RDS deletion protection is not enabled. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.", [r.address])
}

deny_rds_del_protection[msg]{
	not array_contains(warning_mode, "rds_del_protection")
	rds_del_protection[_] != []
	msg := rds_del_protection[_]
}

warn_rds_del_protection[msg]{
	array_contains(warning_mode, "rds_del_protection")
	rds_del_protection[_] != []
	msg := rds_del_protection[_]
}

# === Deny if RDS is publicly accessible ===
rds_publicly_accessible[reason] {
	r := rds_instances[_]
	r.change.after.publicly_accessible == true
	reason := sprintf("%s: RDS is publicly accessible. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.", [r.name])
}

deny_rds_publicly_accessible[msg]{
	not array_contains(warning_mode, "rds_publicly_accessible")
	rds_publicly_accessible[_] != []
	msg := rds_publicly_accessible[_]
}

warn_rds_publicly_accessible[msg]{
	array_contains(warning_mode, "rds_publicly_accessible")
	rds_publicly_accessible[_] != []
	msg := rds_publicly_accessible[_]
}

# === Deny if RDS is not encrypted ===
rds_not_encrypted[reason] {
	r := rds_instances[_]
	not r.change.after.storage_encrypted == true
	reason := sprintf("%s: RDS storage is not encrypted. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.", [r.name])
}

deny_rds_not_encrypted[msg]{
	not array_contains(warning_mode, "rds_not_encrypted")
	rds_not_encrypted[_] != []
	msg := rds_not_encrypted[_]
}

warn_rds_not_encrypted[msg]{
	array_contains(warning_mode, "rds_not_encrypted")
	rds_not_encrypted[_] != []
	msg := rds_not_encrypted[_]
}

# === Deny if RDS is encrypted but with an AWS owned KMS key ===
rds_encrypted_with_aws_owned_kms[reason]{
	c := input.configuration.root_module.resources[_] # Get the changes from the root module configuration
	c.type == "aws_db_instance" # Check that it's a RDS instance
    # Check that encryption is enabled
    c.expressions.storage_encrypted.constant_value == true
	# Check the expressions for a KMS key. If false, then it's using an AWS owned KMS key.
	kms_key := object.get(c.expressions,"kms_key_id",false)
	kms_key == false
	reason := sprintf("%s: RDS storage is encrypted with AWS owned KMS key. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.", [c.name])
}

deny_rds_encrypted_with_aws_owned_kms[msg]{
	not array_contains(warning_mode, "rds_encrypted_with_aws_owned_kms")
	rds_encrypted_with_aws_owned_kms[_] != []
	msg := rds_encrypted_with_aws_owned_kms[_]
}

warn_rds_encrypted_with_aws_owned_kms[msg]{
	array_contains(warning_mode, "rds_encrypted_with_aws_owned_kms")
	rds_encrypted_with_aws_owned_kms[_] != []
	msg := rds_encrypted_with_aws_owned_kms[_]
}

# === Deny if RDS auto minor version upgrade is not enabeled ===
rds_auto_upgrade[reason] {
	r := rds_instances[_]
	not r.change.after.auto_minor_version_upgrade == true
	reason := sprintf("%s: RDS auto minor version upgrade is not enabled. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.", [r.address])
}

deny_rds_auto_upgrade[msg]{
	not array_contains(warning_mode, "rds_auto_upgrade")
	rds_auto_upgrade[_] != []
	msg := rds_auto_upgrade[_]
}

warn_rds_auto_upgrade[msg]{
	array_contains(warning_mode, "rds_auto_upgrade")
	rds_auto_upgrade[_] != []
	msg := rds_auto_upgrade[_]
}
