package main

s3_required_tags = {"data_classification"}
allowed_acls = ["private","log-delivery-write"]
allowed_sse_algorithms = ["aws:kms", "AES256"]

s3_buckets[r] {
    r := input.resource_changes[_]
    r.type == "aws_s3_bucket"
}

# Get Terrform Plan Changeset
module_address[i] = address {
	changeset := input.resource_changes[i]
	address := changeset.address
}

s3_tags_contain_proper_keys(tags) {
	keys := {key | tags[key]}
	leftover := s3_required_tags - keys
	leftover == set()
}

s3_tags_contain_minimum_set[i] = resources {
	changeset := input.resource_changes[i]
    changeset.type == "aws_s3_bucket"
	tags := changeset.change.after.tags
	resources := [resource | resource := module_address[i]; not s3_tags_contain_proper_keys(changeset.change.after.tags)]
}

# Commenting out access logging rules but leaving here just in case opinions change and
# it is needed -- kashley 12/2/2021
#
# has_logging_enabled(r) {
# 	r.change.after_unknown.logging == []
# } else {
#     enabled := r.change.after_unknown.logging[_].target_bucket
# 	not enabled == true
# }

################################################################################
## Test Cases                                                                 ##
################################################################################

# === Check S3 bucket tags ===
missing_s3_tags[msg] {
	resources := s3_tags_contain_minimum_set[_]
	resources != []
	msg := sprintf("Missing one or more of the minimum required tags %v for the following resources: %v. \nPlease see SBU tagging policy for more information https://confluence.carbonblack.local/display/cbengx/SBU+AWS+Tagging+Policy \nAdditional information: https://source.vmware.com/intranet/document/5974/I-12_2.pdf", [s3_required_tags, resources])
}

deny_missing_s3_tags[msg]{
  not array_contains(warning_mode, "missing_s3_tags")
  missing_s3_tags[_] != []
  msg := missing_s3_tags[_]
}

warn_missing_s3_tags[msg]{
  array_contains(warning_mode, "missing_s3_tags")
  missing_s3_tags[_] != []
  msg := missing_s3_tags[_]
}

# === Rule to restrict S3 bucket ACLs ===
bucket_acl[reason] {
    r := s3_buckets[_]
    not array_contains(allowed_acls, r.change.after.acl)
    reason := sprintf(
        "%s: ACL %q is not allowed. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address, r.change.after.acl]
    )
}

deny_bucket_acl[msg]{
  not array_contains(warning_mode, "bucket_acl")
  bucket_acl[_] != []
  msg := bucket_acl[_]
}

warn_bucket_acl[msg]{
  array_contains(warning_mode, "bucket_acl")
  bucket_acl[_] != []
  msg := bucket_acl[_]
}

# === Rule to require server-side encryption ===
bucket_server_side_encryption[reason] {
    r := s3_buckets[_]
    count(r.change.after.server_side_encryption_configuration) == 0
    reason := sprintf(
        "%s: requires server-side encryption with expected sse_algorithm to be one of %v. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address, allowed_sse_algorithms]
    )
}

deny_bucket_server_side_encryption[msg]{
  not array_contains(warning_mode, "bucket_server_side_encryption")
  bucket_server_side_encryption[_] != []
  msg := bucket_server_side_encryption[_]
}

warn_bucket_server_side_encryption[msg]{
  array_contains(warning_mode, "bucket_server_side_encryption")
  bucket_server_side_encryption[_] != []
  msg := bucket_server_side_encryption[_]
}

# === Rule to enforce specific SSE algorithms ===
bucket_sse_algorithm[reason] {
    r := s3_buckets[_]
    sse_configuration := r.change.after.server_side_encryption_configuration[_]
    apply_sse_by_default := sse_configuration.rule[_].apply_server_side_encryption_by_default[_]
    not array_contains(allowed_sse_algorithms, apply_sse_by_default.sse_algorithm)
    reason := sprintf(
        "%s: expected sse_algorithm to be one of %v. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address, allowed_sse_algorithms]
    )
}

deny_bucket_sse_algorithm[msg]{
  not array_contains(warning_mode, "bucket_sse_algorithm")
  bucket_sse_algorithm[_] != []
  msg := bucket_sse_algorithm[_]
}

warn_bucket_sse_algorithm[msg]{
  array_contains(warning_mode, "bucket_sse_algorithm")
  bucket_sse_algorithm[_] != []
  msg := bucket_sse_algorithm[_]
}

# === Deny if versioning is missing ===
buckets_without_versioning[msg] {
    r := s3_buckets[_]
    bucket_name := r.address # Get current bucket name
    versioning := r.change.after.versioning[_] # Get current bucket versioning element
    value := object.get(versioning, "enabled", false) # Check if the enabled key exists
    not value == true # Check that the value of the enabled key is true
    msg = sprintf("S3 Bucket %v doesn't have versioning enabled, See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.", [bucket_name])
}

#buckets_without_versioning[msg] {
#    r := s3_buckets[_]
#    not r.change.after.versioning.enabled == true
#    bucket_name := s3_buckets[_].address
#    msg = sprintf("S3 Bucket %v doesn't have versioning enabled, See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.", [bucket_name])
#}

deny_buckets_without_versioning[msg]{
  not array_contains(warning_mode, "buckets_without_versioning")
  buckets_without_versioning[_] != []
  msg := buckets_without_versioning[_]
}

warn_buckets_without_versioning[msg]{
  array_contains(warning_mode, "buckets_without_versioning")
  buckets_without_versioning[_] != []
  msg := buckets_without_versioning[_]
}

# Commenting out access logging rules but leaving here just in case opinions change and
# it is needed -- kashley 12/2/2021
#
# # === Rule to require S3 bucket access logging ===
# bucket_logging[reason] {
#     r := s3_buckets[_]
#     not array_contains(["log-delivery-write"], r.change.after.acl) # exclude log buckets from requiring logging
#     has_logging_enabled(r)
#     reason := sprintf(
#         "%s: Server Access Logging is suggest for %s buckets. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
#         [r.address, r.change.after.acl]
#     )
# }

# deny_bucket_logging[msg]{
#   not array_contains(warning_mode, "bucket_logging")
#   bucket_logging[_] != []
#   msg := bucket_logging[_]
# }

# warn_bucket_logging[msg]{
#   array_contains(warning_mode, "bucket_logging")
#   bucket_logging[_] != []
#   msg := bucket_logging[_]
# }

# === Rule to require lifecycle policies on logging buckets be greater than a set number of days ===
bucket_log_retention[reason] {
    r := s3_buckets[_]
    log_retention_period := 90 # number of days to use as the test for retention
    array_contains(["log-delivery-write"], r.change.after.acl) # if logging bucket
    rules := r.change.after.lifecycle_rule[_];
    rules.enabled == true # if rule is enabled
    rules.expiration[_].days < log_retention_period # if retention is less than required
    reason := sprintf(
        "%s: Lifecycle rule '%s' detected on a logging bucket with a retention of %d days, which is less than the required %d day retention. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address, rules.id, rules.expiration[_].days, log_retention_period]
    )
}

deny_bucket_log_retention[msg]{
  not array_contains(warning_mode, "bucket_log_retention")
  bucket_log_retention[_] != []
  msg := bucket_log_retention[_]
}

warn_bucket_log_retention[msg]{
  array_contains(warning_mode, "bucket_log_retention")
  bucket_log_retention[_] != []
  msg := bucket_log_retention[_]
}